use super::SVGUnit;
use std::cmp::Ordering;
use std::fmt;
use std::{
    fmt::Display,
    ops::{Add, Div, Mul, Sub},
};

#[derive(Debug, Clone, Copy)]
pub struct SVGMeasure {
    pub measure: f64,
    pub unit: SVGUnit,
}

impl SVGMeasure {
    pub fn new(measure: f64, unit: SVGUnit) -> Self {
        Self { measure, unit }
    }

    fn to_user_units(&self) -> f64 {
        self.measure * self.unit.to_user_units()
    }

    fn to_unit(&self, unit: SVGUnit) -> Self {
        SVGMeasure {
            measure: self.to_user_units() / unit.to_user_units(),
            unit,
        }
    }
}

const EQ_TOLERANCE: f64 = 0.00001;

impl PartialEq for SVGMeasure {
    fn eq(&self, other: &Self) -> bool {
        (self.to_user_units() - other.to_user_units()).abs() < EQ_TOLERANCE
    }

    fn ne(&self, other: &Self) -> bool {
        (self.to_user_units() - other.to_user_units()).abs() > EQ_TOLERANCE
    }
}

impl PartialOrd for SVGMeasure {
    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
        Some(if self == other {
            Ordering::Equal
        } else if self.to_user_units() > other.to_user_units() {
            Ordering::Greater
        } else {
            Ordering::Less
        })
    }
}

impl Display for SVGMeasure {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        write!(f, "{}{}", self.measure, self.unit)
    }
}

impl Add for SVGMeasure {
    type Output = Self;

    fn add(mut self, other: Self) -> Self {
        self.measure += other.to_unit(self.unit).measure;
        self
    }
}

impl Sub for SVGMeasure {
    type Output = Self;

    fn sub(mut self, other: Self) -> Self {
        self.measure -= other.to_unit(self.unit).measure;
        self
    }
}

impl Mul<f64> for SVGMeasure {
    type Output = Self;

    fn mul(mut self, scalar: f64) -> Self {
        self.measure *= scalar;
        self
    }
}

impl Div<f64> for SVGMeasure {
    type Output = Self;

    fn div(mut self, scalar: f64) -> Self {
        self.measure /= scalar;
        self
    }
}
